package com.feedback.demo.service;

import com.feedback.demo.config.CustomUserDetail;
import com.feedback.demo.dto.login.ChangePasswordRequest;
import com.feedback.demo.dto.login.LoginResponse;
import com.feedback.demo.dto.login.SignupRequest;
import com.feedback.demo.entities.FeedBackOptions;
import com.feedback.demo.entities.FeedBackSettings;
import com.feedback.demo.entities.User;
import com.feedback.demo.exceptionhandle.ApiMissingException;
import com.feedback.demo.exceptionhandle.UsernameIsAlreadyTakenException;
import com.feedback.demo.repositories.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
@Transactional(rollbackFor = Exception.class)
public class UserService {
    @Autowired
    private UserRepository userRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    private JwtService jwtService;

    public User login(String username, String password) {
        Authentication authentication = authenticationManager
                .authenticate(new UsernamePasswordAuthenticationToken(username, password));
        SecurityContextHolder.getContext().setAuthentication(authentication);
        return getCurrentUser();
    }

    public User getCurrentUser() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication instanceof AnonymousAuthenticationToken) {
            throw new ApiMissingException();
        }
        CustomUserDetail customUserDetail = (CustomUserDetail) authentication.getPrincipal();
        return customUserDetail.getUser();
    }

    public LoginResponse checkToken(String token) throws IOException {
        User user = jwtService.getUserFromToken(token);
        if (user != null) {
            String jwt = jwtService.generateToken(new CustomUserDetail(user));
            return new LoginResponse(user.getId(), user.getUsername(), jwt);
        } else {
            throw new RuntimeException("Token is valid");
        }
    }

    public LoginResponse getLoginResponse(String username, String password) {
        User user = login(username, password);
        String jwt = jwtService.generateToken(new CustomUserDetail(user));
        return new LoginResponse(user.getId(), user.getUsername(), jwt);
    }

    public LoginResponse changePassword(ChangePasswordRequest changePasswordRequest) {
        User user = getCurrentUser();
        if (passwordEncoder.matches(changePasswordRequest.getOldPass(), user.getPassword())) {
            user.setPassword(passwordEncoder.encode(changePasswordRequest.getNewPass()));
            userRepository.save(user);
            String jwt = jwtService.generateToken(new CustomUserDetail(user));
            return new LoginResponse(user.getId(), user.getUsername(), jwt);
        } else {
            throw new RuntimeException("Old password not match !");
        }
    }

    public LoginResponse signUp(SignupRequest signupRequest) {
        User userCheck = userRepository.findByUsername(signupRequest.getUsername());
        if (userCheck != null) {
            throw new UsernameIsAlreadyTakenException();
        }
        User user = User.builder()
                .username(signupRequest.getUsername())
                .password(
                        passwordEncoder.encode(signupRequest.getPassword())
                )
                .email(signupRequest.getEmail())
                .phone(signupRequest.getPhone())
                .domain(signupRequest.getDomain())
                .feedBacks(null)
                .build();
        FeedBackSettings defaultSettings = getDefaultSettings();
        defaultSettings.setUser(user);
        user.setFeedBackSettings(defaultSettings);
        List<FeedBackOptions> defaultOptions = getDefaultOptions(user);
        user.setFeedBackOptions(defaultOptions);
        userRepository.save(user);
        return getLoginResponse(signupRequest.getUsername(),signupRequest.getPassword());
    }
    public FeedBackSettings getDefaultSettings(){
        return FeedBackSettings.builder()
                .position(1)
                .feedbackTitle("Hệ thống gửi phản hồi")
                .titleFullName("Họ và tên")
                .titleEmail("Email")
                .titlePhoneNumber("Số điện thoại")
                .titleOptions("Danh mục phải hồi")
                .titleDescription("Nội dung chi tiết")
                .iconURL(null)
                .build();
    }
    public List<FeedBackOptions> getDefaultOptions(User user){
        List<FeedBackOptions> feedBackOptions = new ArrayList<>();
        feedBackOptions.add(FeedBackOptions.builder()
                .optionOrder(1)
                .optionValue("Sản phẩm rất tốt")
                .user(user)
                .build());
        feedBackOptions.add(FeedBackOptions.builder()
                .optionOrder(2)
                .optionValue("Sản phẩm tốt")
                .user(user)
                .build());
        feedBackOptions.add(FeedBackOptions.builder()
                .optionOrder(3)
                .optionValue("Sản phẩm bình thường")
                .user(user)
                .build());
        feedBackOptions.add(FeedBackOptions.builder()
                .optionOrder(4)
                .optionValue("Sản phẩm kém")
                .user(user)
                .build());
        feedBackOptions.add(FeedBackOptions.builder()
                .optionOrder(5)
                .optionValue("Sản phẩm quá kém")
                .user(user)
                .build());
        return feedBackOptions;
    }
}
